@@ -1,4 +1,4 @@
-#!/usr/bin/env python3
+#!/usr/bin/env python
 # License: GPL v3 Copyright: 2018, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -7,7 +7,7 @@ import os
 import sys
 from contextlib import contextmanager
 from functools import partial
-from typing import TYPE_CHECKING, Any, Dict, FrozenSet, Generator, List, cast
+from typing import TYPE_CHECKING, Any, Dict, FrozenSet, Generator, List, Optional, cast
 
 from kitty.constants import list_kitty_resources
 from kitty.types import run_once
@@ -59,7 +59,10 @@ def import_kitten_main_module(config_dir: str, kitten: str) -> Dict[str, Any]:
 
     kitten = resolved_kitten(kitten)
     m = importlib.import_module(f'kittens.{kitten}.main')
-    return {'start': getattr(m, 'main'), 'end': getattr(m, 'handle_result', lambda *a, **k: None)}
+    return {
+        'start': getattr(m, 'main'),
+        'end': getattr(m, 'handle_result', lambda *a, **k: None),
+    }
 
 
 def create_kitten_handler(kitten: str, orig_args: List[str]) -> Any:
@@ -70,6 +73,7 @@ def create_kitten_handler(kitten: str, orig_args: List[str]) -> Any:
     setattr(ans, 'type_of_input', getattr(m['end'], 'type_of_input', None))
     setattr(ans, 'no_ui', getattr(m['end'], 'no_ui', False))
     setattr(ans, 'has_ready_notification', getattr(m['end'], 'has_ready_notification', False))
+    setattr(ans, 'open_url_handler', getattr(m['end'], 'open_url_handler', None))
     return ans
 
 
@@ -171,7 +175,7 @@ def get_kitten_completer(kitten: str) -> Any:
     return ans
 
 
-def get_kitten_conf_docs(kitten: str) -> Definition:
+def get_kitten_conf_docs(kitten: str) -> Optional[Definition]:
     setattr(sys, 'options_definition', None)
     run_kitten(kitten, run_name='__conf__')
     ans = getattr(sys, 'options_definition')
@@ -179,6 +183,14 @@ def get_kitten_conf_docs(kitten: str) -> Definition:
     return cast(Definition, ans)
 
 
+def get_kitten_extra_cli_parsers(kitten: str) -> Dict[str,str]:
+    setattr(sys, 'extra_cli_parsers', {})
+    run_kitten(kitten, run_name='__extra_cli_parsers__')
+    ans = getattr(sys, 'extra_cli_parsers')
+    delattr(sys, 'extra_cli_parsers')
+    return cast(Dict[str, str], ans)
+
+
 def main() -> None:
     try:
         args = sys.argv[1:]
