@@ -12,18 +12,19 @@ import struct
 import sys
 import termios
 import time
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from functools import wraps
 from pty import CHILD, STDIN_FILENO, STDOUT_FILENO, fork
+from typing import Optional
 from unittest import TestCase
 
 from kitty.config import finalize_keys, finalize_mouse_mappings
-from kitty.fast_data_types import Cursor, HistoryBuf, LineBuf, Screen, get_options, set_options
+from kitty.fast_data_types import Cursor, HistoryBuf, LineBuf, Screen, get_options, monotonic, set_options
 from kitty.options.parse import merge_result_dicts
 from kitty.options.types import Options, defaults
 from kitty.types import MouseEvent
 from kitty.utils import read_screen_size
-from kitty.window import process_remote_print, process_title_from_child
+from kitty.window import decode_cmdline, process_remote_print, process_title_from_child
 
 
 def parse_bytes(screen, data, dump_callback=None):
@@ -42,6 +43,9 @@ class Callbacks:
         self.pty = pty
         self.ftc = None
         self.set_pointer_shape = lambda data: None
+        self.last_cmd_at = 0
+        self.last_cmd_cmdline = ''
+        self.last_cmd_exit_status = sys.maxsize
 
     def write(self, data) -> None:
         self.wtcbuf += bytes(data)
@@ -64,8 +68,15 @@ class Callbacks:
     def color_profile_popped(self, x) -> None:
         pass
 
-    def cmd_output_marking(self, is_start: bool) -> None:
-        pass
+    def cmd_output_marking(self, is_start: Optional[bool], data: str = '') -> None:
+        if is_start:
+            self.last_cmd_at = monotonic()
+            self.last_cmd_cmdline = decode_cmdline(data) if data else data
+        else:
+            if self.last_cmd_at != 0:
+                self.last_cmd_at = 0
+                with suppress(Exception):
+                    self.last_cmd_exit_status = int(data)
 
     def request_capabilities(self, q) -> None:
         from kitty.terminfo import get_capabilities
@@ -92,6 +103,9 @@ class Callbacks:
         self.bell_count = 0
         self.clone_cmds = []
         self.current_clone_data = ''
+        self.last_cmd_exit_status = sys.maxsize
+        self.last_cmd_cmdline = ''
+        self.last_cmd_at = 0
 
     def on_bell(self) -> None:
         self.bell_count += 1
@@ -334,38 +348,33 @@ class PTY:
             self.process_input_from_child(0)
 
     def send_cmd_to_child(self, cmd, flush=False):
+        self.callbacks.last_cmd_exit_status = sys.maxsize
+        self.last_cmd = cmd
         self.write_to_child(cmd + '\r', flush=flush)
 
     def process_input_from_child(self, timeout=10):
-        rd, wd, err = select.select([self.master_fd], [self.master_fd] if self.write_buf else [], [], timeout)
-        while wd:
-            try:
-                n = os.write(self.master_fd, self.write_buf)
-            except (BlockingIOError, OSError):
-                n = 0
-            if not n:
-                break
+        rd, wd, _ = select.select([self.master_fd], [self.master_fd] if self.write_buf else [], [], max(0, timeout))
+        if wd:
+            n = os.write(self.master_fd, self.write_buf)
             self.write_buf = self.write_buf[n:]
 
         bytes_read = 0
-        while rd:
-            try:
-                data = os.read(self.master_fd, io.DEFAULT_BUFFER_SIZE)
-            except (BlockingIOError, OSError):
-                data = b''
-            if not data:
-                break
+        if rd:
+            data = os.read(self.master_fd, io.DEFAULT_BUFFER_SIZE)
             bytes_read += len(data)
             self.received_bytes += data
             parse_bytes(self.screen, data)
         return bytes_read
 
-    def wait_till(self, q, timeout=10):
+    def wait_till(self, q, timeout=10, timeout_msg=None):
         end_time = time.monotonic() + timeout
         while not q() and time.monotonic() <= end_time:
-            self.process_input_from_child(timeout=max(0, end_time - time.monotonic()))
+            self.process_input_from_child(timeout=end_time - time.monotonic())
         if not q():
-            raise TimeoutError(f'The condition was not met. Screen contents: \n {repr(self.screen_contents())}')
+            msg = 'The condition was not met'
+            if timeout_msg is not None:
+                msg = timeout_msg()
+            raise TimeoutError(f'Timed out: {msg}. Screen contents: \n {repr(self.screen_contents())}')
 
     def wait_till_child_exits(self, timeout=30 if BaseTest.is_ci else 10, require_exit_code=None):
         end_time = time.monotonic() + timeout
@@ -379,7 +388,8 @@ class PTY:
                         f'Child exited with exit status: {status} code: {ec} != {require_exit_code}.'
                         f' Screen contents:\n{self.screen_contents()}')
                 return status
-            self.process_input_from_child(timeout=0.02)
+            with suppress(OSError):
+                self.process_input_from_child(timeout=0.02)
         raise AssertionError(f'Child did not exit in {timeout} seconds. Screen contents:\n{self.screen_contents()}')
 
     def set_window_size(self, rows=25, columns=80, send_signal=True):
