@@ -438,6 +438,9 @@ class TestKeys(BaseTest):
         ae(tq(ord('a'), action=defines.GLFW_REPEAT), csi(num='a', action=2))
         ae(tq(ord('a'), action=defines.GLFW_RELEASE), csi(num='a', action=3))
         ae(tq(ord('a'), action=defines.GLFW_RELEASE, mods=shift), csi(shift, num='a', action=3))
+        tq = partial(enc, key_encoding_flags=0b11)
+        ae(tq(defines.GLFW_FKEY_BACKSPACE), '\x7f')
+        ae(tq(defines.GLFW_FKEY_BACKSPACE, action=release), '')
 
         # test alternate key reporting
         aq = partial(enc, key_encoding_flags=0b100)
@@ -527,6 +530,7 @@ class TestKeys(BaseTest):
                 bad_lines = []
                 self.options = load_config(overrides=lines, accumulate_bad_lines=bad_lines)
                 af(bad_lines)
+                self.ignore_os_keyboard_processing = False
                 super().__init__()
 
             def get_active_window(self):
@@ -555,7 +559,7 @@ class TestKeys(BaseTest):
                 return bool(action_definition)
 
             def set_ignore_os_keyboard_processing(self, on: bool) -> None:
-                pass
+                self.ignore_os_keyboard_processing = on
 
             def set_cocoa_global_shortcuts(self, opts):
                 return {}
@@ -594,10 +598,29 @@ class TestKeys(BaseTest):
         tm = TM('map kitty_mod+enter')
         self.ae(tm('ctrl+shift+enter'), [False])
 
-        # single key mapping overrides all multi-key mappings with same prefix
+        # single key mapping overrides previous all multi-key mappings with same prefix
         tm = TM('map kitty_mod+p new_window')
         self.ae(tm('ctrl+shift+p', 'f'), [True, False])
         self.ae(tm.actions, ['new_window'])
+        # multi-key mapping overrides previous single key mapping with same prefix
+        tm = TM('map kitty_mod+s>p new_window')
+        self.ae(tm('ctrl+shift+s', 'p'), [True, True])
+        self.ae(tm.actions, ['new_window'])
+        # mix of single and multi-key mappings with same prefix
+        tm = TM('map alt+p>1 multi1', 'map alt+p single1', 'map alt+p>2 multi2')
+        self.ae(tm('alt+p', '2'), [True, True])
+        self.ae(tm.actions, ['multi2'])
+        self.ae(tm('alt+p', '1'), [True, False])
+        af(tm.actions)
+        self.ae(len(tm.active_window.key_seqs), 1)
+
+        # a single multi-key mapping should not prematurely match
+        tm = TM('map alt+1>2>3 new_window')
+        self.ae(tm('alt+1', '2'), [True, True])
+        af(tm.actions)
+        tm = TM('map alt+1>2>3 new_window')
+        self.ae(tm('alt+1', '2', '3'), [True, True, True])
+        self.ae(tm.actions, ['new_window'])
 
         # changing a multi key mapping
         tm = TM('map kitty_mod+p>f new_window')
@@ -621,3 +644,9 @@ class TestKeys(BaseTest):
         self.ae(tm.actions, ['neighboring_window left'])
         self.ae(tm('x'), [True])
         af(tm.keyboard_mode_stack)
+
+        # modal mapping with --on-action=end must restore OS keyboard processing
+        tm = TM('map --new-mode mw --on-action end m', 'map --mode mw a new_window')
+        self.ae(tm('m', 'a'), [True, True])
+        self.ae(tm.actions, ['push_keyboard_mode mw', 'new_window'])
+        af(tm.ignore_os_keyboard_processing)
